@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 
-# Copyright 2013 Facundo Batista
+# Copyright 2013-2014 Facundo Batista
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -44,6 +44,7 @@ from PyQt4.QtGui import (
 from PyQt4.QtCore import Qt, QSize
 
 from encuentro import data, image
+from encuentro.config import config, signal
 from encuentro.data import Status
 from encuentro.ui import remembering
 from encuentro.ui.throbber import Throbber
@@ -57,6 +58,7 @@ class DownloadsWidget(remembering.RememberingTreeWidget):
     def __init__(self, episodes_widget):
         self.episodes_widget = episodes_widget
         super(DownloadsWidget, self).__init__('downloads')
+        signal.register(self.save_state)
 
         _headers = (u"Descargando...", u"Estado")
         self.setColumnCount(len(_headers))
@@ -77,20 +79,21 @@ class DownloadsWidget(remembering.RememberingTreeWidget):
     def append(self, episode):
         """Append an episode to the downloads list."""
         # add to the list in the GUI
-        item = QTreeWidgetItem((episode.title, u"Encolado"))
+        item = QTreeWidgetItem((episode.composed_title, u"Encolado"))
         item.episode_id = episode.episode_id
         self.queue.append((episode, item))
         self.addTopLevelItem(item)
         self.setCurrentItem(item)
 
         # fix episode state
-        episode.state = Status.downloading
+        episode.state = Status.waiting
 
     def prepare(self):
         """Set up everything for next download."""
         self.downloading = True
         self.current += 1
         episode, _ = self.queue[self.current]
+        episode.state = Status.downloading
         return episode
 
     def start(self):
@@ -123,8 +126,28 @@ class DownloadsWidget(remembering.RememberingTreeWidget):
 
     def cancel(self):
         """The download is being cancelled."""
-        _, item = self.queue[self.current]
+        episode, item = self.queue[self.current]
         item.setText(1, u"Cancelado")
+        episode.state = Status.none
+
+    def unqueue(self, episode):
+        """Remove the indicated episode from the queue."""
+        episode.state = Status.none
+
+        # search for the item, adjust the queue and remove it from the widget
+        for pos, (queued_episode, item) in enumerate(self.queue):
+            if queued_episode.episode_id == episode.episode_id:
+                break
+        else:
+            raise ValueError(
+                "Couldn't find episode to unqueue: " + str(episode))
+        del self.queue[pos]
+        self.takeTopLevelItem(pos)
+
+        # as we removed an item, the cursor goes to other, fix the rest of
+        # the interface
+        item = self.currentItem()
+        self.episodes_widget.show(item.episode_id)
 
     def pending(self):
         """Return the pending downloads quantity (including current)."""
@@ -135,6 +158,29 @@ class DownloadsWidget(remembering.RememberingTreeWidget):
             q += 1
         return q
 
+    def save_state(self):
+        """Save state for pending downloads."""
+        p = self.pending()
+        if p > 0:
+            pending_ids = [e.episode_id for e, _ in self.queue[-p:]]
+        else:
+            pending_ids = []
+        config[config.SYSTEM]['pending_ids'] = pending_ids
+
+    def load_pending(self):
+        """Queue the pending downloads."""
+        loaded_pending_ids = config[config.SYSTEM].get('pending_ids', [])
+
+        for episode_id in loaded_pending_ids:
+            main_window = self.episodes_widget.main_window
+            try:
+                episode = main_window.programs_data[episode_id]
+            except KeyError:
+                logger.debug("Tried to load pending %r, didn't find it",
+                             episode_id)
+            else:
+                main_window.queue_download(episode)
+
 
 class HTMLDelegate(QStyledItemDelegate):
     """Custom delegate so the QTreeWidget can do HTML.
@@ -196,7 +242,7 @@ class EpisodesWidget(remembering.RememberingTreeWidget):
     """The list of episodes info."""
 
     _row_getter = operator.attrgetter('channel', 'section',
-                                      'title', 'duration')
+                                      'composed_title', 'duration')
     _title_column = 2
 
     def __init__(self, main_window, episode_info):
@@ -232,6 +278,7 @@ class EpisodesWidget(remembering.RememberingTreeWidget):
         self.setContextMenuPolicy(Qt.CustomContextMenu)
         self.customContextMenuRequested.connect(self.on_right_button)
         self.itemDoubleClicked.connect(self.on_double_click)
+        self.currentItemChanged.connect(self.on_change)
 
     def show(self, episode_id):
         """Show the row for the requested episode."""
@@ -244,6 +291,11 @@ class EpisodesWidget(remembering.RememberingTreeWidget):
         item = self.currentItem()
         self._adjust_gui(item.episode_id)
 
+    def on_change(self, _):
+        """The view was clicked."""
+        item = self.currentItem()
+        self._adjust_gui(item.episode_id)
+
     def on_double_click(self, item, col):
         """Double click."""
         episode = self.main_window.programs_data[item.episode_id]
@@ -326,6 +378,7 @@ class EpisodesWidget(remembering.RememberingTreeWidget):
 
     def set_filter(self, text, only_downloaded=False):
         """Apply a filter to the episodes list."""
+        text = data.prepare_to_filter(text)
         for episode_id, item in self._item_map.iteritems():
             episode = self.main_window.programs_data[episode_id]
 
@@ -337,10 +390,10 @@ class EpisodesWidget(remembering.RememberingTreeWidget):
                 pos1, pos2 = params
                 if pos1 is None:
                     # no highlighting
-                    item.setText(self._title_column, episode.title)
+                    item.setText(self._title_column, episode.composed_title)
                 else:
                     # filtering by text, so highlight
-                    t = episode.title
+                    t = episode.composed_title
                     ht = u''.join((t[:pos1],
                                    '<span style="background-color:yellow">',
                                    t[pos1:pos2], '</span>', t[pos2:]))
@@ -425,8 +478,8 @@ class EpisodeInfo(QWidget):
                            episode.image_url.encode('utf-8'))
 
         # all description
-        msg = "<center><h3>%s</h3></center><br/><br/>%s" % (
-            episode.title, episode.description)
+        msg = u"<center><h3>%s</h3></center><br/><br/>%s" % (
+            episode.composed_title, episode.description)
         self.text_edit.setHtml(msg)
 
         # action button
@@ -435,20 +488,28 @@ class EpisodeInfo(QWidget):
             label = "Reproducir"
             func = self.main_window.play_episode
             enable = True
-        elif (episode.state == data.Status.downloading or
-              episode.state == data.Status.waiting):
+            remove = False
+        elif episode.state == data.Status.downloading:
             label = u"Cancelar descarga"
             func = self.main_window.cancel_download
             enable = True
+            remove = False
+        elif episode.state == data.Status.waiting:
+            label = u"Sacar de la cola"
+            func = self.main_window.unqueue_download
+            enable = True
+            remove = True
         else:
             label = u"Descargar"
             func = self.main_window.download_episode
             enable = bool(self.main_window.have_config())
+            remove = False
 
-        def _exec(func, episode):
+        def _exec(func, episode, remove):
             """Execute a function on the episode and update its info."""
             func(episode)
-            self.update(episode)
+            if not remove:
+                self.update(episode)
 
         # set button text, disconnect if should, and connect new func
         self.button.setEnabled(enable)
@@ -456,7 +517,7 @@ class EpisodeInfo(QWidget):
         if self.button.connected:
             self.button.clicked.disconnect()
         self.button.connected = True
-        self.button.clicked.connect(lambda: _exec(func, episode))
+        self.button.clicked.connect(lambda: _exec(func, episode, remove))
 
 
 class BigPanel(QWidget):
