@@ -1,6 +1,6 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 
-# Copyright 2013 Facundo Batista
+# Copyright 2013-2014 Facundo Batista
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -20,6 +20,9 @@
 
 import logging
 import os
+import datetime as dt
+
+import defer
 
 try:
     import pynotify
@@ -36,14 +39,14 @@ from PyQt4.QtGui import (
     QStyle,
     QWidget,
 )
-from twisted.internet import defer
 
-from encuentro import platform, data, update
+from encuentro import multiplatform, data, update
 from encuentro.config import config, signal
 from encuentro.data import Status
 from encuentro.network import (
     BadCredentialsError,
     CancelledError,
+    EncuentroError,
     all_downloaders,
 )
 from encuentro.ui import (
@@ -83,10 +86,17 @@ Versión %s<br/>
 """
 
 
+def notify(title, message):
+    """Shows a notification (if user wants and have stuff installed)."""
+    if config.get('notification', True) and pynotify is not None:
+        n = pynotify.Notification(title, message)
+        n.show()
+
+
 class MainUI(remembering.RememberingMainWindow):
     """Main UI."""
 
-    _programs_file = os.path.join(platform.data_dir, 'encuentro.data')
+    _programs_file = os.path.join(multiplatform.data_dir, 'encuentro.data')
 
     def __init__(self, version, app_quit):
         super(MainUI, self).__init__()
@@ -115,10 +125,22 @@ class MainUI(remembering.RememberingMainWindow):
         self._menubar()
 
         systray.show(self)
+
         if config.get('autorefresh'):
             ue = update.UpdateEpisodes(self)
             ue.background()
+        else:
+            # refresh data if never done before or if last
+            # update was 7 days ago
+            last_refresh = config.get('autorefresh_last_time')
+            if last_refresh is None or (
+                    dt.datetime.now() - last_refresh > dt.timedelta(7)):
+                ue = update.UpdateEpisodes(self)
+                ue.background()
+
         self.show()
+
+        self.episodes_download.load_pending()
         logger.debug("Main UI started ok")
 
     def _sanitize_config(self):
@@ -138,7 +160,7 @@ class MainUI(remembering.RememberingMainWindow):
 
         # we have a default for download dir
         if not config.get('downloaddir'):
-            config['downloaddir'] = platform.get_download_dir()
+            config['downloaddir'] = multiplatform.get_download_dir()
 
         # maybe clean some config
         if self.programs_data.reset_config_from_migration:
@@ -359,7 +381,7 @@ class MainUI(remembering.RememberingMainWindow):
             episode = self.programs_data[item.episode_id]
             self.queue_download(episode)
 
-    @defer.inlineCallbacks
+    @defer.inline_callbacks
     def queue_download(self, episode):
         """User indicated to download something."""
         logger.debug("Download requested of %s", episode)
@@ -389,10 +411,18 @@ class MainUI(remembering.RememberingMainWindow):
                        u"usuario y clave correctos")
                 self.show_message('BadCredentialsError', msg)
                 self.episodes_download.end(error=msg)
-            except Exception, e:
-                logger.debug("Unknown download error: %s", e)
+            except EncuentroError, e:
+                orig_exc = e.orig_exc
+                msg = "%s(%s)" % (orig_exc, e)
                 err_type = e.__class__.__name__
-                self.show_message(err_type, str(e))
+                logger.exception("Custom Encuentro error: %s (%r)",
+                                 e, orig_exc)
+                notify(err_type, msg)
+                self.episodes_download.end(error=u"Error: " + msg)
+            except Exception, e:
+                logger.exception("Unknown download error: %s (%r)", e, e)
+                err_type = e.__class__.__name__
+                notify(err_type, str(e))
                 self.episodes_download.end(error=u"Error: " + str(e))
             else:
                 logger.debug("Episode downloaded: %s", episode)
@@ -405,7 +435,7 @@ class MainUI(remembering.RememberingMainWindow):
 
         logger.debug("Downloads: finished")
 
-    @defer.inlineCallbacks
+    @defer.inline_callbacks
     def _episode_download(self, episode):
         """Effectively download an episode."""
         logger.debug("Effectively downloading episode %s", episode.episode_id)
@@ -413,16 +443,14 @@ class MainUI(remembering.RememberingMainWindow):
 
         # download!
         downloader = self.downloaders[episode.downtype]
-        fname = yield downloader.download(episode.channel,
-                                          episode.section, episode.title,
-                                          episode.url,
-                                          self.episodes_download.progress)
+        season = getattr(episode, 'season', None)  # wasn't always there
+        fname = yield downloader.download(
+            episode.channel, episode.section, season, episode.title,
+            episode.url, self.episodes_download.progress)
         episode_name = u"%s - %s - %s" % (episode.channel, episode.section,
-                                          episode.title)
-        if config.get('notification', True) and pynotify is not None:
-            n = pynotify.Notification(u"Descarga finalizada", episode_name)
-            n.show()
-        defer.returnValue((fname, episode))
+                                          episode.composed_title)
+        notify(u"Descarga finalizada", episode_name)
+        defer.return_value((fname, episode))
 
     def open_preferences(self, _=None):
         """Open the preferences dialog."""
@@ -485,7 +513,7 @@ class MainUI(remembering.RememberingMainWindow):
             # pass file:// url with absolute path
             fullpath = 'file://' + os.path.abspath(filename)
             logger.info("Playing %r", fullpath)
-            platform.open_file(fullpath)
+            multiplatform.open_file(fullpath)
         else:
             logger.warning("Aborted playing, file not found: %r", filename)
             msg = (u"No se encontró el archivo para reproducir: " +
@@ -500,7 +528,11 @@ class MainUI(remembering.RememberingMainWindow):
         self.episodes_download.cancel()
         downloader = self.downloaders[episode.downtype]
         downloader.cancel()
-        episode.state = Status.none
+
+    def unqueue_download(self, episode):
+        """Remove the episode from the download queue."""
+        logger.info("Unqueueing %s", episode)
+        self.episodes_download.unqueue(episode)
 
     def open_about_dialog(self):
         """Show the about dialog."""
